import json
import os
from typing import Optional, List

import cv2
import ffmpeg
from sd_parsers import ParserManager
from sd_parsers.data import PromptInfo

from pylizlib.core.domain.os import FileType
from pylizlib.core.log.pylizLogger import logger
from pylizlib.core.os.file import get_file_c_date, get_file_type, is_media_file
from pylizlib.media.domain.ai import AiPayloadMediaInfo


class LizMedia:

    def __init__(self, path: str):

        # file info
        self.path = path
        self.file_name = os.path.basename(self.path)
        self.extension = os.path.splitext(path)[1].lower()
        self.creation_time = get_file_c_date(self.path)
        self.creation_time_timestamp: float = self.creation_time.timestamp()
        self.year, self.month, self.day = self.creation_time.year, self.creation_time.month, self.creation_time.day
        self.size_byte = os.path.getsize(self.path)
        self.size_mb = self.size_byte / (1024 * 1024)

        # type of media
        if not is_media_file(self.path):
            raise ValueError(f"File {self.path} is not a media file.")
        self.type = get_file_type(self.path)
        self.is_image = self.type == FileType.IMAGE
        self.is_video = self.type == FileType.VIDEO
        self.is_audio = self.type == FileType.AUDIO

        # ai info
        self.ai_ocr_text: Optional[List[str]] = None
        self.ai_file_name: Optional[str] = None
        self.ai_description: Optional[str] = None
        self.ai_tags: Optional[List[str]] = None
        self.ai_scanned: bool = False
        self.ai_generated: bool = False
        self.ai_metadata: PromptInfo | None = None
        self.__check_for_ai_metadata()

        # video info
        if self.is_video:
            self.duration_sec: Optional[float] = self.get_video_duration_seconds()
            self.duration_min: Optional[float] = self.duration_sec / 60
            self.frame_rate: Optional[float] = self.get_video_frame_rate()
        else:
            self.duration_sec = None
            self.duration_min = None
            self.frame_rate = None

    def __check_for_ai_metadata(self):
        try:
            if self.is_image:
                parser_manager = ParserManager()
                prompt_info: PromptInfo | None = parser_manager.parse(self.path)
                if prompt_info is not None:
                    self.ai_metadata = prompt_info
                    self.ai_generated = True
        except Exception as e:
            logger.error(f"Error checking for AI metadata with sdParser: {str(e)}")


    def get_desc_plus_text(self):
        if self.ai_ocr_text is not None and len(self.ai_ocr_text) > 0:
            text_array = []
            for text in self.ai_ocr_text:
                text_array.append(text)
            return self.ai_description + " This media includes texts: " + " ".join(text_array)
        return self.ai_description

    def get_video_duration_seconds(self) -> float:
        try:
            probe = ffmpeg.probe(self.path)
            duration = float(probe['format']['duration'])
            return duration
        except Exception as e:
            logger.error(f"Error getting video duration: {str(e)}")
        return 0.0

    def get_video_frame_rate(self) -> float:
        """
        Restituisce il frame rate (FPS) del video.
        """
        try:
            video = cv2.VideoCapture(self.path)
            if not video.isOpened():
                logger.error("Errore: impossibile aprire il video.")
                return 0.0

            # Ottieni il frame rate
            fps = video.get(cv2.CAP_PROP_FPS)
            video.release()

            # Ritorna il valore di FPS
            return fps
        except Exception as e:
            logger.error(f"Errore nel calcolo del frame rate del video: {e}")
            return 0.0

    def to_dict_only_ai(self):
        return {
            "path": self.path,
            "file_name": self.file_name,
            "extension": self.extension,
            "creation_time_timestamp": self.creation_time_timestamp,
            "size_byte": self.size_byte,
            "size_mb": self.size_mb,
            "ai_ocr_text": self.ai_ocr_text,
            "ai_file_name": self.ai_file_name,
            "ai_description": self.ai_description,
            "ai_tags": self.ai_tags,
            "ai_scanned": self.ai_scanned,
        }

    def to_json_only_ai(self):
        return json.dumps(self.to_dict_only_ai(), indent=4)

    def apply_ai_info(self, ai_info: AiPayloadMediaInfo):
        self.ai_ocr_text = ai_info.text
        self.ai_file_name = ai_info.filename
        self.ai_description = ai_info.description
        self.ai_tags = ai_info.tags
        self.ai_scanned = True




import logging
import os
from datetime import datetime

# Global logger instances
rootConsoleLogger = logging.getLogger(__name__ + ".console")
rootFileLogger = logging.getLogger(__name__ + ".file")

# Default values for Loggiz
default_console_log_level = logging.DEBUG
default_console_log_format = "[%(levelname)s]: %(message)s"
default_file_log_level = logging.DEBUG
default_file_log_format = "%(asctime)s - [%(levelname)s]: %(message)s"
default_tag_char_start = "("
default_tag_char_end = "): "


class Loggiz:
    @staticmethod
    def setup(
            app_name="app",
            setup_console=True,
            setup_file=True,
            console_level=default_console_log_level,
            console_format=default_console_log_format,
            console_ansi=False,
            file_level=default_file_log_level,
            file_format=default_file_log_format,
            file_log_base_path="",
            file_log_folder_name="logs",
            file_log_name="latest.log",
            use_app_timestamp_template=True,
    ):
        if setup_file:
            if use_app_timestamp_template:
                file_log_name = Loggiz.create_timestamp_log_file_name(app_name)
            else:
                file_log_name = file_log_name
            log_folder_dir = os.path.join(file_log_base_path, file_log_folder_name)
            file_full_path = os.path.join(log_folder_dir, file_log_name)
            if not os.path.exists(log_folder_dir):
                os.makedirs(log_folder_dir)
            config.set_file_log_path(file_full_path)
            config.set_logger_file_enabled(True)
            Loggiz.setup_file(
                level=file_level,
                log_format=file_format,
                file_path=file_full_path,
            )

        if setup_console:
            Loggiz.setup_console(
                level=console_level,
                log_format=console_format,
            )
            config.set_console_ansi(console_ansi)
            config.set_logger_console_enabled(True)


    @staticmethod
    def setup_console(level=default_console_log_level, log_format=default_console_log_format):
        rootConsoleLogger.setLevel(level)
        console_handler = logging.StreamHandler()
        console_formatter = logging.Formatter(log_format)
        console_handler.setFormatter(console_formatter)
        rootConsoleLogger.addHandler(console_handler)

    @staticmethod
    def setup_file(level=default_file_log_level, log_format=default_file_log_format, file_path="loggiz.log"):
        rootFileLogger.setLevel(level)
        file_handler = logging.FileHandler(file_path)
        file_formatter = logging.Formatter(log_format)
        file_handler.setFormatter(file_formatter)
        rootFileLogger.addHandler(file_handler)

    @staticmethod
    def create_timestamp_log_file_name(app_name):
        timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
        log_file_name = f"{app_name}_{timestamp}.log"
        return log_file_name

    @staticmethod
    def create_timestamp_log_file_path(app_name, log_dir):
        log_file_name = Loggiz.create_timestamp_log_file_name(app_name)
        log_file_path = os.path.join(log_dir, log_file_name)
        return log_file_path


# Config class for Loggiz
class LoggizConfig:
    def __init__(self):
        self.enable_file_ansi = False
        self.enable_console_ansi = False
        self.tag_char_start = default_tag_char_start
        self.tag_char_end = default_tag_char_end
        self.file_log_path = ""
        self.logger_console_enabled = False
        self.logger_file_enabled = False

    def set_console_ansi(self, ansi):
        self.enable_console_ansi = ansi

    def set_file_ansi(self, ansi):
        self.enable_file_ansi = ansi

    def set_tag_chars(self, start, end):
        self.tag_char_start = start
        self.tag_char_end = end

    def set_file_log_path(self, path):
        self.file_log_path = path

    def set_logger_console_enabled(self, enabled):
        self.logger_console_enabled = enabled

    def set_logger_file_enabled(self, enabled):
        self.logger_file_enabled = enabled


# ANSI color codes
class Colors:
    BLACK = "\033[0;30m"
    RED = "\033[0;31m"
    GREEN = "\033[0;32m"
    BROWN = "\033[0;33m"
    BLUE = "\033[0;34m"
    PURPLE = "\033[0;35m"
    CYAN = "\033[0;36m"
    LIGHT_GRAY = "\033[0;37m"
    DARK_GRAY = "\033[1;30m"
    LIGHT_RED = "\033[1;31m"
    LIGHT_GREEN = "\033[1;32m"
    YELLOW = "\033[1;33m"
    LIGHT_BLUE = "\033[1;34m"
    LIGHT_PURPLE = "\033[1;35m"
    LIGHT_CYAN = "\033[1;36m"
    LIGHT_WHITE = "\033[1;37m"
    BOLD = "\033[1m"
    FAINT = "\033[2m"
    ITALIC = "\033[3m"
    UNDERLINE = "\033[4m"
    BLINK = "\033[5m"
    NEGATIVE = "\033[7m"
    CROSSED = "\033[9m"
    END = "\033[0m"
    if not __import__("sys").stdout.isatty():
        for _ in dir():
            if isinstance(_, str) and _[0] != "_":
                locals()[_] = ""
    else:
        if __import__("platform").system() == "Windows":
            kernel32 = __import__("ctypes").windll.kernel32
            kernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)
            del kernel32


config = LoggizConfig()  # Current configuration for Loggiz


class LoggizLogger:

    @staticmethod
    def get_tag_string(tag, ansify=False):
        if ansify:
            return f"{Colors.LIGHT_WHITE}{config.tag_char_start}{tag}{config.tag_char_end}{Colors.END}"
        else:
            return f"{config.tag_char_start}{tag}{config.tag_char_end}"

    @staticmethod
    def print_separator(sep, count):
        string = f'{sep}' * count
        if config.enable_console_ansi:
            print(Colors.LIGHT_GRAY + string + Colors.END)
        else:
            print(string)

    # ------------------------ CONSOLE LOGGING ------------------------

    @staticmethod
    def log_console_debug(tag, message):
        if config.enable_console_ansi:
            rootConsoleLogger.debug(LoggizLogger.get_tag_string(tag, True) + Colors.CYAN + f"{message}" + Colors.END)
        else:
            rootConsoleLogger.debug(LoggizLogger.get_tag_string(tag) + message)

    @staticmethod
    def log_console_info(tag, message):
        if config.enable_console_ansi:
            rootConsoleLogger.info(LoggizLogger.get_tag_string(tag, True) + Colors.GREEN + f"{message}" + Colors.END)
        else:
            rootConsoleLogger.info(LoggizLogger.get_tag_string(tag) + message)

    @staticmethod
    def log_console_warning(tag, message):
        if config.enable_console_ansi:
            rootConsoleLogger.warning(LoggizLogger.get_tag_string(tag, True) + Colors.YELLOW + f"{message}" + Colors.END)
        else:
            rootConsoleLogger.warning(LoggizLogger.get_tag_string(tag) + message)

    @staticmethod
    def log_console_error(tag, message):
        if config.enable_console_ansi:
            rootConsoleLogger.error(LoggizLogger.get_tag_string(tag, True) + Colors.RED + f"{message}" + Colors.END)
        else:
            rootConsoleLogger.error(LoggizLogger.get_tag_string(tag) + message)

    # ------------------------ FILE LOGGING ------------------------

    @staticmethod
    def log_file_debug(tag, message):
        rootFileLogger.debug(LoggizLogger.get_tag_string(tag) + message)

    @staticmethod
    def log_file_info(tag, message):
        rootFileLogger.info(LoggizLogger.get_tag_string(tag) + message)

    @staticmethod
    def log_file_warning(tag, message):
        rootFileLogger.warning(LoggizLogger.get_tag_string(tag) + message)

    @staticmethod
    def log_file_error(tag, message):
        rootFileLogger.error(LoggizLogger.get_tag_string(tag) + message)

    # ------------------------  LOGGING METHODS ------------------------

    @staticmethod
    def debug_tag(tag, message):
        if config.logger_console_enabled:
            LoggizLogger.log_console_debug(tag, message)
        if config.logger_file_enabled:
            LoggizLogger.log_file_debug(tag, message)

    @staticmethod
    def info_tag(tag, message):
        if config.logger_console_enabled:
            LoggizLogger.log_console_info(tag, message)
        if config.logger_file_enabled:
            LoggizLogger.log_file_info(tag, message)

    @staticmethod
    def warning_tag(tag, message):
        if config.logger_console_enabled:
            LoggizLogger.log_console_warning(tag, message)
        if config.logger_file_enabled:
            LoggizLogger.log_file_warning(tag, message)

    @staticmethod
    def error_tag(tag, message):
        if config.logger_console_enabled:
            LoggizLogger.log_console_error(tag, message)
        if config.logger_file_enabled:
            LoggizLogger.log_file_error(tag, message)







from dataclasses import dataclass

from qfluentwidgets.common import ConfigItem


class QtFwQConfigItem(ConfigItem):

    def __init__(self, enabled: bool, group, name, default, validator=None, serializer=None, restart=False):
        super().__init__(group, name, default, validator, serializer, restart)
        self.enabled = enabled


@dataclass
class QtFwQConfigGroup:
    name: str
    description: str = ""
    enabled: bool = True


from dataclasses import dataclass
from enum import Enum
from typing import TypeVar, Any

T = TypeVar("T")


@dataclass
class OperationInfo:
    name: str
    description: str
    delay_each_task: float = 0.0


class OperationStatus(Enum):
    Pending = "Pending"
    InProgress = "In Progress"
    Completed = "Completed"
    Failed = "Failed"



from typing import Generic, TypeVar, Optional

T = TypeVar('T')


class Operation(Generic[T]):
    def __init__(self, payload: Optional[T] = None, status: bool = False, error: Optional[str] = None):
        self.payload = payload
        self.status = status
        self.error = error

    # @classmethod
    # def is_ok(cls, operation: 'Operation[T]') -> bool:
    #     return operation.status

    def is_op_ok(self) -> bool:
        return self.status

    def __str__(self):
        return f"Operation(status={self.status}, payload={self.payload}, error={self.error})"




import os
from dataclasses import asdict
from pathlib import Path

from qfluentwidgets import ConfigValidator, ConfigSerializer, ConfigItem



class ExecutableValidator(ConfigValidator):
    """Executable file validator"""

    def validate(self, value):
        path = Path(value)
        # Controlla che il file esista ed è eseguibile
        return path.is_file() and os.access(path, os.X_OK)

    def correct(self, value):
        path = Path(value)
        # Se il file non esiste o non è eseguibile, ritorna valore senza modifiche
        if not path.exists() or not os.access(path, os.X_OK):
            return value
        # Ritorna il percorso assoluto con slashes forward
        return str(path.resolve()).replace("\\", "/")

class TextListValidator(ConfigValidator):
    """Text list validator - check non-empty strings"""

    def validate(self, value):
        # value is expected to be a list of strings
        return all(isinstance(i, str) and i.strip() != "" for i in value)

    def correct(self, value: list[str]):
        # Rimuove voci vuote o solo spazi
        return [i.strip() for i in value if isinstance(i, str) and i.strip() != ""]


def count_elements_in_settings_keys(data, key, group=None):
    total_count = 0

    groups_to_check = [group] if group else data.keys()

    for grp in groups_to_check:
        group_data = data.get(grp, {})
        if isinstance(group_data, dict) and key in group_data and isinstance(group_data[key], list):
            total_count += len(group_data[key])

    return total_count




from functools import partial
from pathlib import Path
from typing import Any

from qfluentwidgets import SubtitleLabel, setFont, TransparentDropDownPushButton, DropDownPushButton, RoundMenu, \
    Dialog, FluentIcon, Action


class UiUtils:

    @staticmethod
    def show_message(title, text: str, parent=None):
        w = Dialog(title, text, parent)
        w.cancelButton.hide()
        w.buttonLayout.insertStretch(1)
        w.exec()

    @staticmethod
    def create_widget_act_bar_btn(
            parent: Any,
            data: list[str] | list[Path],
            btn_name: str,
            icon: FluentIcon,
            is_trasparent: bool = True,
            clicked_signal=None,
            icon_action: FluentIcon | None = None
    ):
        actions: list[Action] = []
        for action_text in data:
            if isinstance(action_text, Path):
                action_text = action_text.__str__()
            act = Action(icon if icon_action is None else icon_action, action_text, parent)
            if clicked_signal is not None:
                act.triggered.connect(partial(clicked_signal.emit, action_text))
            actions.append(act)

        if is_trasparent:
            button = TransparentDropDownPushButton(btn_name, parent, icon)
        else:
            button = DropDownPushButton(btn_name, parent, icon)
        button.setFixedHeight(34)
        setFont(button, 12)

        menu = RoundMenu(parent=parent)
        for act in actions:
            menu.addAction(act)
        button.setMenu(menu)

        return button




def convert_byte_to_mb(byte: int) -> float:
    return byte / (1024 * 1024)


def get_total_sec_from_msec(msec: int) -> int:
    return msec // 1000


def get_sec60_from_msec(msec: int) -> int:
    return get_total_sec_from_msec(msec) % 60


def get_min_from_msec(msec: int) -> int:
    return get_total_sec_from_msec(msec) // 60


def convert_months_number_to_str(number: int) -> str:
    months = {
        1: "January",
        2: "February",
        3: "March",
        4: "April",
        5: "May",
        6: "June",
        7: "July",
        8: "August",
        9: "September",
        10: "October",
        11: "November",
        12: "December"
    }
    return months.get(number, "Invalid Month")

def get_normalized_gb_mb_str(total_size: float ) -> str:
    size_mb = total_size / (1024 * 1024)
    if size_mb < 1000:
        return f"{size_mb:.2f} MB"
    size_gb = size_mb / 1024
    return f"{size_gb:.2f} GB"